@@ -20,6 +20,7 @@
 
 #include <GeoIP.h>
 #include <GeoIPCity.h>
+#include <GeoIP_internal.h>
 
 void usage() {
 	fprintf(stderr,"Usage: geoiplookup [-v] <ipaddress|hostname>\n");
@@ -29,6 +30,7 @@ int main (int argc, char *argv[]) {
 	const char * hostname;
 	const char * country_code;
 	const char * country_name;
+	int country_id;
 	GeoIPRegion * region;
 	GeoIPRecord * gir;
 	const char * org;
@@ -56,7 +58,7 @@ int main (int argc, char *argv[]) {
 		hostname = argv[1];
 	}
 
-	_setup_dbfilename();
+	_GeoIP_setup_dbfilename();
 
 	/* iterate through different database types */
 	for (i = 0; i < NUM_DB_TYPES; ++i) {
@@ -66,8 +68,9 @@ int main (int argc, char *argv[]) {
 				printf("%s not available, skipping...\n", GeoIPDBDescription[i]);
 			} else {
 				if (GEOIP_COUNTRY_EDITION == i) {
-					country_code = GeoIP_country_code_by_name(gi, hostname);
-					country_name = GeoIP_country_name_by_name(gi, hostname);
+					country_id = GeoIP_id_by_name(gi, hostname);
+					country_code = GeoIP_country_code[country_id];
+					country_name = GeoIP_country_name[country_id];
 					if (country_code == NULL) {
 						printf("%s: IP Address not found\n", GeoIPDBDescription[i]);
 					} else {
