@@ -21,6 +21,7 @@
 #include "GeoIPCity.h"
 #include "GeoIP.h"
 #include "GeoIPUpdate.h"
+#include "GeoIP_internal.h"
 
 #include "global.h"
 #include "md5.h"
@@ -39,9 +40,6 @@
 #include <stdio.h>
 #include <unistd.h>
 
-extern void _setup_dbfilename();
-extern char *_full_path_to(const char *file_name);
-
 const char *GeoIPUpdateHost = "updates.maxmind.com";
 const char *GeoIPHTTPRequest = "GET /app/update?license_key=%s&md5=%s HTTP/1.0\nHost: updates.maxmind.com\n\n";
 const char *GeoIPHTTPRequestMD5 = "GET /app/update_secure?db_md5=%s&challenge_md5=%s&user_id=%s&edition_id=%s HTTP/1.0\nHost: updates.maxmind.com\n\n";
@@ -54,6 +52,49 @@ const char *WritingFile = "Writing uncompressed data to %s ...";
 
 /* TODO replace printf with GeoIP_printf - we need someway of having vargs with GeoIP_printf */
 
+const char * GeoIP_get_error_message(int i) {
+  switch (i) {
+  case GEOIP_NO_NEW_UPDATES:
+    return "no new updates";
+  case GEOIP_SUCCESS:
+    return "Success";
+  case GEOIP_LICENSE_KEY_INVALID_ERR:
+    return "License Key Invalid";
+  case GEOIP_DNS_ERR:
+    return "Unable to resolve hostname";
+  case GEOIP_NON_IPV4_ERR:
+    return "Non - IPv4 address";
+  case GEOIP_SOCKET_OPEN_ERR:
+    return "Error opening socket";
+  case GEOIP_CONNECTION_ERR:
+    return "Unable to connect";
+  case GEOIP_GZIP_IO_ERR:
+    return "Unable to write GeoIP.dat.gz file";
+  case GEOIP_TEST_IO_ERR:
+    return "Unable to write GeoIP.dat.test file";
+  case GEOIP_GZIP_READ_ERR:
+    return "Unable to read gzip data";
+  case GEOIP_OUT_OF_MEMORY_ERR:
+    return "Out of memory error";
+  case GEOIP_SOCKET_READ_ERR:
+    return "Error reading from socket, see errno";
+  case GEOIP_SANITY_OPEN_ERR:
+    return "Sanity check GeoIP_open error";
+  case GEOIP_SANITY_INFO_FAIL:
+    return "Sanity check database_info string failed";
+  case GEOIP_SANITY_LOOKUP_FAIL:
+    return "Sanity check ip address lookup failed";
+  case GEOIP_RENAME_ERR:
+    return "Rename error while installing db, check errno";
+  case GEOIP_USER_ID_INVALID_ERR:
+    return "Invalid userID";
+  case GEOIP_PRODUCT_ID_INVALID_ERR:
+    return "Invalid product ID or subscription expired";
+  default:
+    return "no error";
+  }  
+}
+
 void GeoIP_printf(void (*f)(char *), const char *str) {
 	char * f_str;
 	f_str = malloc(strlen(str)+1);
@@ -83,7 +124,7 @@ short int GeoIP_update_database (char * license_key, int verbose, void (*f)( cha
 	GeoIP * gi;
 	char * db_info;
 
-	_setup_dbfilename();
+	_GeoIP_setup_dbfilename();
 
 	/* get MD5 of current GeoIP database file */
 	if ((cur_db_fh = fopen (GeoIPDBFileName[GEOIP_COUNTRY_EDITION], "rb")) == NULL) {
@@ -385,14 +426,14 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 	buf[offset] = 0;
 	offset = 0;
 	tmpstr = strstr(buf, "\r\n\r\n") + 4;
-	geoipfilename = _full_path_to(tmpstr);
+	geoipfilename = _GeoIP_full_path_to(tmpstr);
 	free(buf);
 
 	/* print the database product id and the database filename */
 	if (verbose == 1){
 		printf("database product id %s database file name %s \n",data_base_type,geoipfilename);
 	}
-	_setup_dbfilename();
+	_GeoIP_setup_dbfilename();
 
 	/* get MD5 of current GeoIP database file */
 	if ((cur_db_fh = fopen (geoipfilename, "rb")) == NULL) {
@@ -739,7 +780,7 @@ short int GeoIP_update_database_general (char * user_id,char * license_key,char
 			(dbtype == GEOIP_ORG_EDITION)) {
 		/* if data base type is isp or org then call the function
 		 * named GeoIP_org_by_addr */
-		GeoIPRecord *r = GeoIP_org_by_addr(gi,"24.24.24.24");
+		GeoIPRecord *r = (GeoIPRecord*)GeoIP_org_by_addr(gi,"24.24.24.24");
 		lookupresult = 0;
 		if (r != NULL) {
 			lookupresult = 1;
